#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>
#include <float.h> /* for DBL_MAX */

#include "multinest.h"

#define num_rows 100
#define num_cols 3
#define num_pars 2

/* data needs to be in the global scope */
double data[num_rows][num_cols];

/* read data from a text file */
void read_data()
{
	FILE* data_file;
	int i,j;
	char* file_name="../line_data.dat";
	char* dummy=(char*)malloc(128*sizeof(char));
	
	/* read data */
	data_file=fopen(file_name,"r");
	if(data_file!=NULL)
	{
		for(i=0;i<num_rows;++i)
		{
			for(j=0;j<num_cols;++j)
			{
				fscanf(data_file,"%s",dummy);
				data[i][j]=atof(dummy);
			}
			
			/*printf("%d\t%f\t%f\t%f\n",i,data[i][0],data[i][1],data[i][2]);*/
		}
		fclose(data_file);
		free(dummy);
	}
}

/* equation of the line */
double line_model(double x,double m,double c)
{
	return m*x+c;
}

/* prior */
double flat_prior(double r, double x1, double x2)
{
	return x1 + r * ( x2 - x1 );
}


/* likelihood */
void log_like(double *cube, int *ndim, int *npars, double *lnew, void *context)
{
	double m,c,val;
	int i;
	double chi2=0;
	double obs,pred,sig,x;
	
	/* The model is y=m*x + c 
	   The parameters are m=0.7 and c=2.5
	   Gaussian error with mu=0 and sigma=1 
	   has bee added to y to create data */
	
	/* apply flat prior with range (-10,10) on m */
	val=flat_prior(cube[0], -10., 10.);
	cube[0]=m=val;
	
	/* apply flat prior with range (-50,50) on c */
	val=flat_prior(cube[1], -50., 50.);
	cube[1]=c=val;
	
	for(i=0;i<num_rows;++i)
	{
		x=data[i][0];
		pred=line_model(x,m,c);
		obs=data[i][1];
		sig=data[i][2];
		
		chi2+=(obs-pred)*(obs-pred)/(sig*sig);
	}	
	
	*lnew = -0.5*chi2;
}

/* For MultiNest */
void dumper(int *nSamples, int *nlive, int *nPar, double **physLive, double **posterior, double **paramConstr, double *maxLogLike, double *logZ, double *logZerr, void *context)
{
}

int main(void)
{
	int i;
	
	// set the MultiNest sampling parameters
	
	
	int mmodal = 1;				// do mode separation?
	
	int ceff = 0;				// run in constant efficiency mode?
	
	int nlive = 1000;			// number of live points
	
	double efr = 1.0;			// set the required efficiency
	
	double tol = 0.5;			// tol, defines the stopping criteria
	
	int ndims = 2;				// dimensionality (no. of free parameters)
	
	int nPar = 2;				// total no. of parameters including free & derived parameters
	
	int nClsPar = 2;			// no. of parameters to do mode separation on
	
	int updInt = 100;			// after how many iterations feedback is required 
						// & the output files should be updated
						// note: posterior files are updated & dumper routine 
						// is called after every updInt*10 iterations
	
	double Ztol = -1E90;			// all the modes with logZ < Ztol are ignored
	
	int maxModes = 100;			// expected max no. of modes (used only for memory allocation)
	
	int pWrap[ndims];			// which parameters to have periodic boundary conditions?
	for(i = 0; i < ndims; i++) pWrap[i] = 0;
	
	char root[100] = "chains/fit_a_line_";	// root for output files
	
	int seed = -1;				// random no. generator seed, if < 0 then take the seed from system clock
	
	int fb = 1;				// need feedback on standard output?
	
	int resume = 0;				// resume from a previous job?
	
	int outfile = 1;			// write output files?
	
	int initMPI = 1;			// initialize MPI routines?, relevant only if compiling with MPI
						// set it to F if you want your main program to handle MPI initialization
	
	double logZero = -DBL_MAX;		// points with loglike < logZero will be ignored by MultiNest
	
	int maxiter = 0;			// max no. of iterations, a non-positive value means infinity. 
						// MultiNest will terminate if either it 
						// has done max no. of iterations or convergence criterion 
						// (defined through tol) has been satisfied
	
	void *context = 0;			// not required by MultiNest, any additional information user wants to pass
	
	
	
	/* read the data */
	read_data();

	
	
	// calling MultiNest	
	run(mmodal, ceff, nlive, tol, efr, ndims, nPar, nClsPar, maxModes, updInt, Ztol, root, seed, pWrap, fb, resume, outfile, initMPI, 
	logZero, maxiter, log_like, dumper, context);
	
	return 0;
}
