import numpy as np
import matplotlib.pyplot as plt

# true values
true_m=0.7
true_c=2.5

#read posterior samples
post=np.loadtxt("./build/chains/fit_a_line_post_equal_weights.dat")
data=np.loadtxt("./line_data.dat")

#calculate the mean of the posterior samples
mean_m=np.mean(post[:,0])
mean_c=np.mean(post[:,1])

#data
x=data[:,0]
ydata=data[:,1]
errbars=data[:,2]

#estimated signal
y=mean_m*x+mean_c

#true signal
true_y=true_m*x+true_c

#make plots
plt.errorbar(x,ydata,yerr=errbars,fmt='o',color='gray',label='data')
plt.plot(x,y,'b',label='estimate')
plt.plot(x,true_y,'r',label='true')
plt.legend(loc=0)
plt.xlabel('x')
plt.ylabel('y')
plt.savefig("./plots/best_fit_line.png")

#plot the marginal posterior distribution of m
plt.clf()
plt.hist(post[:,0],histtype='step',bins=20,color='k',label='samples')
ax=plt.axis()
plt.plot([true_m,true_m],[ax[2],ax[3]],color='r',label='true value')
plt.xlabel("m")
plt.ylabel('count')
plt.legend(loc=0)
plt.savefig("./plots/post_m.png")
#plt.show()

#plot the marginal posterior distribution of c
plt.clf()
plt.hist(post[:,1],histtype='step',bins=20,color='k',label='samples')
ax=plt.axis()
plt.plot([true_c,true_c],[ax[2],ax[3]],color='r',label='true value')
plt.xlabel("c")
plt.ylabel('count')
plt.legend(loc=0)
plt.savefig("./plots/post_c.png")
#plt.show()





